@@ -1,6 +1,7 @@
 # coding: utf-8
-import select
+import select, threading
 from pylibcurl import Multi, Curl, const
+from Queue import Queue, Empty
 
 event_epoll2curl = {
     select.EPOLLIN: const.CURL_CSELECT_IN,
@@ -44,7 +45,10 @@ event_curl2poll = {
 }
 
 
-class Pool(object):
+class Pool(threading.Thread):
+    """
+        Pool work as like Thread and Queue
+    """
     default_curl_settings = dict(
         useragent='Mozilla 6', 
         autoreferer=1, 
@@ -53,9 +57,11 @@ class Pool(object):
         encoding='',
     )
 
-    def __init__(self, maxconnects, *args, **kwargs):
-        self._multi = Multi(*args, **kwargs)
+    def __init__(self, maxconnects, **kwargs):
+        self._multi = Multi(**kwargs)
         self._multi.maxconnects = maxconnects
+        self._queue = Queue()
+        self._size = maxconnects
 
         self._data = {}
         self._poll = select.poll()
@@ -81,45 +87,79 @@ class Pool(object):
         self._multi.socketfunction = socket_cb
         self._multi.timerfunction = timer_cb
 
-    def _socket_action(self, socket, event=0):
-        code, running = self._multi.socket_action(socket, event)
+        super(Pool, self).__init__()
 
+    def _do_socket_action(self, socket, event=0):
+        code = const.CURLM_CALL_MULTI_SOCKET
+        while code == const.CURLM_CALL_MULTI_SOCKET:
+            code, running = self._multi.socket_action(socket, event)
+        
+        return running
+
+    def _do_remove(self):
         # remove complete curls and sockets
         for curl, sock in self._complete:
             self._multi.remove_handle(curl)
             self._poll.unregister(sock)
             self._sockets.remove(sock)
+            
+            # call callback
+            url, header, body, callback = self._data[curl]
+            body = ''.join(body)
 
-        self._complete.clear()
+            if self._Thread__started.is_set():
+                del self._data[curl]
+            else:
+                self._data[curl][2] = body
 
-        return code, running
+            if callback:
+                callback(curl, url, header, body)
+            
+            # task done
+            self._queue.task_done()
+
+        removed = len(self._complete) > 0
+        if removed:
+            self._complete.clear()
+
+        return removed
+
+    
+    def _do_add(self):
+        start_event = self._Thread__started
+        added = False
+
+        while len(self._multi._handles) < self._size:
+            block = False
+            
+            # thread mode and daemon and empty
+            if self.isDaemon() and start_event.is_set() and not self._multi._handles: 
+                block = True
+            
+            try:
+                data = self._queue.get(block=block)
+            except Empty:
+                break
+            else:
+                added = True
+                url, kwargs, callback = data
+                c = self._create_curl(url, callback, **kwargs)
+                self._multi.add_handle(c)
 
 
-    def _run(self):
-        code = const.CURLM_CALL_MULTI_SOCKET
-        while code == const.CURLM_CALL_MULTI_SOCKET:
-            code, running = self._multi.socket_action(const.CURL_SOCKET_TIMEOUT, 0)
-        
-        #return 
-        while running:
-            events = self._poll.poll(1)
-            for sock, event in events:
-                mask = event_poll2curl.get(event, 0)
+        if added:
+            code = const.CURLM_CALL_MULTI_SOCKET
+            while code:
+                code, running = self._multi.socket_action(const.CURL_SOCKET_TIMEOUT, 0)
 
-                if event == select.POLLHUP:
-                    self._poll.unregister(sock)
-                else:
-                    code = const.CURLM_CALL_MULTI_SOCKET
-                    while code == const.CURLM_CALL_MULTI_SOCKET:
-                        code, running = self._socket_action(sock, mask)
+        return added
 
-    def add(self, url, **kwargs):
+    def _create_curl(self, url, callback, **kwargs):
         settings = self.default_curl_settings.copy()
         settings.update(kwargs)
 
         header = []
         body = []
-        error = None
 
         def headerfunction(v):
             v = v.strip()
@@ -134,22 +174,66 @@ class Pool(object):
 
         
         c = Curl(url, **settings)
-        self._data[c] = (url, header, body, error)
-        self._multi.add_handle(c)
+        self._data[c] = [url, header, body, callback]
+        return c
 
-    # TODO: async mode
-    def perform(self, async=False):
-        """
-            perform multi curl
-            if not async:
-                return list of tuple(url, headers, content, error)
-        """
 
-        if async:
-            return 
-        else:
-            self._run()
-            data = [(url, header, ''.join(body), error) 
-                for url, header, body, error in self._data.values()]
+
+    def run(self):
+        """
+            for threading mode use "start" method
+            for non-threading mode use "run" method
+        """
+        start_event = self._Thread__started
+        self._do_add()
+        running = 1
+
+        while True:
+            
+            while running:
+                events = self._poll.poll(1)
+                for sock, event in events:
+                    mask = event_poll2curl.get(event, 0)
+
+                    if event == select.POLLHUP:
+                        self._poll.unregister(sock)
+                    else:
+                        running = self._do_socket_action(sock, mask)
+                        self._do_remove()
+
+                self._do_add()
+
+            # check loop
+            started = start_event.is_set()
+            if not started and not self.isDaemon():
+                break
+
+        # for non-threading mode
+        if not start_event.is_set():
+            data = [(curl, v[0], v[1], v[2]) 
+                for curl, v in self._data.items()]
             self._data.clear()
             return data
+
+    def add(self, url, callback=None, **kwargs):
+        """
+            put values to queue
+
+            callback prototype:
+                def callback(curl, start_url, header, body):
+                    [code]
+        """
+        self._queue.put((url, kwargs, callback))
+
+
+    def join(self, timeout=None):
+        """
+            if thread is daemon then run loop forever
+        """
+        if self.isDaemon():
+            self._queue.join()
+        else:
+            super(Pool, self).join(timeout)
+
+
+
